@@ -12,11 +12,23 @@ fn main() {
     let cc;
     let mut cflags;
 
+    #[cfg(not(feature = "riot-rs"))]
+    if env::var("BUILDING_RIOT_RS").is_ok() {
+        println!("");
+        println!("ERROR: riot-sys seems to be built for RIOT-rs (BUILDING_RIOT_RS is set). Please enable the 'riot-rs' feature.");
+        println!(
+            "To do this, make the main application crate depend on `riot-sys` with feature `riot-rs`."
+        );
+        println!("");
+        std::process::exit(1);
+    }
+
     #[cfg(not(feature = "riot-rs"))]
     let compile_commands_json = "RIOT_COMPILE_COMMANDS_JSON";
     #[cfg(feature = "riot-rs")]
     let compile_commands_json = "DEP_RIOT_BUILD_COMPILE_COMMANDS_JSON";
 
+    println!("cargo:rerun-if-env-changed=BUILDING_RIOT_RS");
     println!("cargo:rerun-if-env-changed=RIOT_CC");
     println!("cargo:rerun-if-env-changed=RIOT_CFLAGS");
     println!("cargo:rerun-if-env-changed={}", &compile_commands_json);
@@ -203,14 +215,21 @@ fn main() {
     // These constant initializers are unusable without knowledge of which type they're for; adding
     // the information here to build explicit consts
     let macro_functions = [
-        ("SOCK_IPV4_EP_ANY", "sock_udp_ep_t", "void", true),
-        ("SOCK_IPV6_EP_ANY", "sock_udp_ep_t", "void", true),
-        ("MUTEX_INIT", "mutex_t", "void", true),
+        ("SOCK_IPV4_EP_ANY", "sock_udp_ep_t", None, true),
+        ("SOCK_IPV6_EP_ANY", "sock_udp_ep_t", None, true),
+        ("MUTEX_INIT", "mutex_t", None, true),
         // neither C2Rust nor bindgen understand the cast without help
-        ("STATUS_NOT_FOUND", "thread_status_t", "void", true),
+        ("STATUS_NOT_FOUND", "thread_status_t", None, true),
         // If any board is ever added that works completely differently, this'll have to go behind
         // a feature-gate
-        ("GPIO_PIN", "gpio_t", "unsigned port, unsigned pin", true),
+        (
+            "GPIO_PIN",
+            "gpio_t",
+            Some("unsigned port, unsigned pin"),
+            // would be nice to have them const, but on boards like samd21-xpro that'd require
+            // several nightly features (const_ptr_offset, const_mut_refs).
+            false,
+        ),
     ];
     let mut macro_functions: Vec<_> = macro_functions
         .iter()
@@ -219,9 +238,9 @@ fn main() {
         })
         .collect();
     for i in 0..8 {
-        macro_functions.push((format!("LED{}_ON", i), "void", "void", false));
-        macro_functions.push((format!("LED{}_OFF", i), "void", "void", false));
-        macro_functions.push((format!("LED{}_TOGGLE", i), "void", "void", false));
+        macro_functions.push((format!("LED{}_ON", i), "void", None, false));
+        macro_functions.push((format!("LED{}_OFF", i), "void", None, false));
+        macro_functions.push((format!("LED{}_TOGGLE", i), "void", None, false));
     }
 
     let mut c_code = String::new();
@@ -231,6 +250,20 @@ fn main() {
         .expect("Failed to read riot-c2rust.h");
 
     for (macro_name, return_type, args, _is_const) in macro_functions.iter() {
+        let argnames = match args {
+            None => "".to_string(),
+            Some("void") => "()".to_string(),
+            Some(args) => format!(
+                "({})",
+                args.split(", ")
+                    .map(|s| &s[s.find(" ").expect("Non-void args need to have names")..])
+                    // Not really essential concepturally, but .join is only available on
+                    // slices, not on Iterator
+                    .collect::<Vec<&str>>()
+                    .join(", ")
+            ),
+        };
+
         // The ifdef guards make errors easier to spot: A "cannot find function
         // `macro_SOCK_IPV6_EP_ANY` in crate `riot_sys`" can lead one to check whether
         // SOCK_IPV6_EP_ANY is really defined, whereas if the macro is missing, C2Rust would
@@ -246,13 +279,14 @@ fn main() {
 
 #ifdef {macro_name}
 {return_type} macro_{macro_name}({args}) {{
-    {macro_name};
+    {macro_name}{argnames};
 }}
 #endif
                 ",
                 return_type = return_type,
                 macro_name = macro_name,
-                args = args,
+                args = args.unwrap_or("void"),
+                argnames = argnames,
             )
         } else {
             write!(
@@ -261,14 +295,15 @@ fn main() {
 
 #ifdef {macro_name}
 {return_type} macro_{macro_name}({args}) {{
-    {return_type} result = {macro_name};
+    {return_type} result = {macro_name}{argnames};
     return result;
 }}
 #endif
                 ",
                 return_type = return_type,
                 macro_name = macro_name,
-                args = args,
+                args = args.unwrap_or("void"),
+                argnames = argnames,
             )
         }
         .unwrap();
@@ -309,10 +344,21 @@ fn main() {
     let compile_commands_name = compile_commands_name
         .to_str()
         .expect("Inexpressible path name");
+
+    println!("cargo:rerun-if-env-changed=C2RUST");
+    println!("cargo:rerun-if-env-changed=PATH");
+    let c2rust = std::env::var("C2RUST").unwrap_or_else(|_| "c2rust".to_string());
+    let c2rust_version = std::process::Command::new(&c2rust)
+        .args(&["--version"])
+        .output()
+        .expect("C2Rust version check did not complete")
+        .stdout;
+    let c2rust_version = String::from_utf8_lossy(&c2rust_version);
+    print!("C2Rust binary {}, version: {}", c2rust, c2rust_version);
     // FIXME: This does not rat on the used files. Most are probably included from riot-bindgen.h
     // anyway, tough.
     println!("Running C2Rust on {}", compile_commands_name);
-    let status = std::process::Command::new("c2rust")
+    let status = std::process::Command::new(&c2rust)
         .args(&[
             "transpile",
             compile_commands_name,
@@ -346,8 +392,10 @@ fn main() {
 
     rustcode = rustcode.replace("use ::libc;\n", "");
 
-    // C2Rust still generates old-style ASM -- workaround for https://github.com/immunant/c2rust/issues/306
-    rustcode = rustcode.replace(" asm!(", " llvm_asm!(");
+    if !c2rust_version.contains("+git-for-riot") && c2rust_version.contains("C2Rust 0.15") {
+        // Old C2Rust still generate old-style ASM -- workaround for https://github.com/immunant/c2rust/issues/306
+        rustcode = rustcode.replace(" asm!(", " llvm_asm!(");
+    }
 
     // Workaround for https://github.com/immunant/c2rust/issues/372
     rustcode = rustcode.replace("::core::intrinsics::", "crate::intrinsics_replacements::");
@@ -363,9 +411,6 @@ fn main() {
         rustcode = rustcode.replace("\n    pub type __lock;", "");
     }
 
-    // This only matches when c2rust is built to even export body-less functions
-    rustcode = rustcode.replace("    #[no_mangle]\n    fn ", "    #[no_mangle]\n    pub fn ");
-
     // Replace the function declarations with ... usually something pub, but special considerations
     // may apply
     let mut rustcode_functionsreplaced = String::new();
@@ -489,7 +534,11 @@ fn main() {
     let toplevel_from_inline: Vec<String> = toplevel_from_inline
         .drain(..)
         .filter(|s: &String| {
-            rustcode.contains(&format!(" {}(", s)) || rustcode.contains(&format!(" {}: ", s))
+            // Not just matching on `pub fn`: `irq_disable` on native is visible as `extern "C" {
+            // fn irq_disable(); }`, and that should not trigger going through C2Rust.
+            rustcode.contains(&format!("pub fn {}(", s))
+                || rustcode.contains(&format!("unsafe fn {}(", s))
+                || rustcode.contains(&format!(" {}: ", s))
         })
         .collect();
     let toplevel_from_inline_filename = out_path.join("toplevel_from_inline.rs");
@@ -551,6 +600,10 @@ fn main() {
             println!("cargo:MARKER_{}=1", name);
         }
     }
+
+    // let downstream crates know we're building for riot-rs
+    #[cfg(feature = "riot-rs")]
+    println!("cargo:MARKER_riot_rs=1");
 }
 
 #[cfg(feature = "riot-rs")]
